# -*- coding: utf-8 -*-
"""
# Clay

An amazing prototyping tool

------------
© [Juan-Pablo Scaletti](http://jpscaletti.com).
[MIT License](http://www.opensource.org/licenses/mit-license.php)

"""
from .main import Clay  # noqa


__version__ = '2.12.4'
